@@ -322,6 +322,34 @@ class MatchersExtractTest(UnitTest):
         )
         self.assertEqual(nodes, {})
 
+    def test_extract_optional_wildcard_head(self) -> None:
+        expression = cst.parse_expression("[3]")
+        nodes = m.extract(
+            expression,
+            m.List(
+                elements=[
+                    m.SaveMatchedNode(m.ZeroOrMore(), "head1"),
+                    m.SaveMatchedNode(m.ZeroOrMore(), "head2"),
+                    m.Element(value=m.Integer(value="3")),
+                ]
+            ),
+        )
+        self.assertEqual(nodes, {"head1": (), "head2": ()})
+
+    def test_extract_optional_wildcard_tail(self) -> None:
+        expression = cst.parse_expression("[3]")
+        nodes = m.extract(
+            expression,
+            m.List(
+                elements=[
+                    m.Element(value=m.Integer(value="3")),
+                    m.SaveMatchedNode(m.ZeroOrMore(), "tail1"),
+                    m.SaveMatchedNode(m.ZeroOrMore(), "tail2"),
+                ]
+            ),
+        )
+        self.assertEqual(nodes, {"tail1": (), "tail2": ()})
+
     def test_extract_optional_wildcard_present(self) -> None:
         expression = cst.parse_expression("a + b[c], d(e, f * g, h.i.j)")
         nodes = m.extract(
@@ -404,3 +432,25 @@ class MatchersExtractTest(UnitTest):
             ),
         )
         self.assertIsNone(nodes)
+
+    def test_extract_sequence_multiple_wildcards(self) -> None:
+        expression = cst.parse_expression("1, 2, 3, 4")
+        nodes = m.extract(
+            expression,
+            m.Tuple(
+                elements=(
+                    m.SaveMatchedNode(m.ZeroOrMore(), "head"),
+                    m.SaveMatchedNode(m.Element(value=m.Integer(value="3")), "element"),
+                    m.SaveMatchedNode(m.ZeroOrMore(), "tail"),
+                )
+            ),
+        )
+        tuple_elements = cst.ensure_type(expression, cst.Tuple).elements
+        self.assertEqual(
+            nodes,
+            {
+                "head": tuple(tuple_elements[:2]),
+                "element": tuple_elements[2],
+                "tail": tuple(tuple_elements[3:]),
+            },
+        )
