@@ -27,7 +27,6 @@ from typing import (
 import libcst as cst
 from libcst import ensure_type
 from libcst._add_slots import add_slots
-from libcst._metadata_dependent import MetadataDependent
 from libcst.helpers import get_full_name_for_node
 from libcst.metadata.base_provider import BatchableMetadataProvider
 from libcst.metadata.expression_context_provider import (
@@ -286,17 +285,26 @@ class _NameUtil:
             return _NameUtil.get_name_for(node.name)
         return None
 
+    @staticmethod
+    def get_module_name_for_import_alike(
+        assignment_node: Union[cst.Import, cst.ImportFrom]
+    ) -> str:
+        module = ""
+        if isinstance(assignment_node, cst.ImportFrom):
+            module_attr = assignment_node.module
+            relative = assignment_node.relative
+            if module_attr:
+                module = get_full_name_for_node(module_attr) or ""
+            if relative:
+                module = "." * len(relative) + module
+        return module
+
     @staticmethod
     def find_qualified_name_for_import_alike(
         assignment_node: Union[cst.Import, cst.ImportFrom], full_name: str
     ) -> Set[QualifiedName]:
-        module = ""
+        module = _NameUtil.get_module_name_for_import_alike(assignment_node)
         results = set()
-        if isinstance(assignment_node, cst.ImportFrom):
-            module_attr = assignment_node.module
-            if module_attr:
-                # TODO: for relative import, keep the relative Dot in the qualified name
-                module = get_full_name_for_node(module_attr)
         import_names = assignment_node.names
         if not isinstance(import_names, cst.ImportStar):
             for name in import_names:
@@ -309,7 +317,11 @@ class _NameUtil:
                 real_names = [".".join(parts[:i]) for i in range(len(parts), 0, -1)]
                 for real_name in real_names:
                     as_name = real_name
-                    if module:
+                    if module and module.endswith("."):
+                        # from . import a
+                        # real_name should be ".a"
+                        real_name = f"{module}{real_name}"
+                    elif module:
                         real_name = f"{module}.{real_name}"
                     if name and name.asname:
                         eval_alias = name.evaluated_alias
@@ -339,13 +351,12 @@ class _NameUtil:
                 name_prefixes.append(scope.name)
             elif isinstance(scope, FunctionScope):
                 name_prefixes.append(f"{scope.name}.<locals>")
-            elif isinstance(scope, GlobalScope):
-                break
             elif isinstance(scope, ComprehensionScope):
                 name_prefixes.append("<comprehension>")
-            else:
+            elif not isinstance(scope, (GlobalScope, BuiltinScope)):
                 raise Exception(f"Unexpected Scope: {scope}")
-            scope = scope.parent
+
+            scope = scope.parent if scope.parent != scope else None
 
         parts = [*reversed(name_prefixes)]
         if remaining_name:
@@ -537,27 +548,57 @@ class Scope(abc.ABC):
         return Accesses(self._accesses)
 
 
+class BuiltinScope(Scope):
+    """
+    A BuiltinScope represents python builtin declarations. See https://docs.python.org/3/library/builtins.html
+    """
+
+    def __init__(self, globals: Scope) -> None:
+        self.globals: Scope = globals  # must be defined before Scope.__init__ is called
+        super().__init__(parent=self)
+
+    def __contains__(self, name: str) -> bool:
+        return hasattr(builtins, name)
+
+    def __getitem__(self, name: str) -> Set[BaseAssignment]:
+        if name in self._assignments:
+            return self._assignments[name]
+        if hasattr(builtins, name):
+            # note - we only see the builtin assignments during the deferred
+            # access resolution. unfortunately that means we have to create the
+            # assignment here, which can cause the set to mutate during iteration
+            self._assignments[name].add(BuiltinAssignment(name, self))
+            return self._assignments[name]
+        return set()
+
+    def record_assignment(self, name: str, node: cst.CSTNode) -> None:
+        raise NotImplementedError("assignments in builtin scope are not allowed")
+
+    def record_global_overwrite(self, name: str) -> None:
+        raise NotImplementedError("global overwrite in builtin scope are not allowed")
+
+    def record_nonlocal_overwrite(self, name: str) -> None:
+        raise NotImplementedError("declarations in builtin scope are not allowed")
+
+
 class GlobalScope(Scope):
     """
     A GlobalScope is the scope of module. All module level assignments are recorded in GlobalScope.
     """
 
     def __init__(self) -> None:
-        self.globals: Scope = self  # must be defined before Scope.__init__ is called
-        super().__init__(parent=self)
+        super().__init__(parent=BuiltinScope(self))
 
     def __contains__(self, name: str) -> bool:
-        return hasattr(builtins, name) or (
-            name in self._assignments and len(self._assignments[name]) > 0
-        )
+        if name in self._assignments:
+            return len(self._assignments[name]) > 0
+        return self.parent._contains_in_self_or_parent(name)
 
     def __getitem__(self, name: str) -> Set[BaseAssignment]:
-        if hasattr(builtins, name):
-            if not any(
-                isinstance(i, BuiltinAssignment) for i in self._assignments[name]
-            ):
-                self._assignments[name].add(BuiltinAssignment(name, self))
-        return self._assignments[name]
+        if name in self._assignments:
+            return self._assignments[name]
+        else:
+            return self.parent._getitem_from_self_or_parent(name)
 
     def record_global_overwrite(self, name: str) -> None:
         pass
@@ -1051,70 +1092,3 @@ class ScopeProvider(BatchableMetadataProvider[Optional[Scope]]):
         visitor = ScopeVisitor(self)
         node.visit(visitor)
         visitor.infer_accesses()
-
-
-class QualifiedNameVisitor(cst.CSTVisitor):
-    def __init__(self, provider: "QualifiedNameProvider") -> None:
-        self.provider: QualifiedNameProvider = provider
-
-    def on_visit(self, node: cst.CSTNode) -> bool:
-        scope = self.provider.get_metadata(ScopeProvider, node, None)
-        if scope:
-            self.provider.set_metadata(node, scope.get_qualified_names_for(node))
-        else:
-            self.provider.set_metadata(node, set())
-        super().on_visit(node)
-        return True
-
-
-class QualifiedNameProvider(BatchableMetadataProvider[Collection[QualifiedName]]):
-    """
-    Compute possible qualified names of a variable CSTNode
-    (extends `PEP-3155 <https://www.python.org/dev/peps/pep-3155/>`_).
-    It uses the
-    :func:`~libcst.metadata.Scope.get_qualified_names_for` underlying to get qualified names.
-    Multiple qualified names may be returned, such as when we have conditional imports or an
-    import shadows another. E.g., the provider finds ``a.b``, ``d.e`` and
-    ``f.g`` as possible qualified names of ``c``::
-
-        >>> wrapper = MetadataWrapper(
-        >>>     cst.parse_module(dedent(
-        >>>     '''
-        >>>         if something:
-        >>>             from a import b as c
-        >>>         elif otherthing:
-        >>>             from d import e as c
-        >>>         else:
-        >>>             from f import g as c
-        >>>         c()
-        >>>     '''
-        >>>     ))
-        >>> )
-        >>> call = wrapper.module.body[1].body[0].value
-        >>> wrapper.resolve(QualifiedNameProvider)[call],
-        {
-            QualifiedName(name="a.b", source=QualifiedNameSource.IMPORT),
-            QualifiedName(name="d.e", source=QualifiedNameSource.IMPORT),
-            QualifiedName(name="f.g", source=QualifiedNameSource.IMPORT),
-        }
-
-    For qualified name of a variable in a function or a comprehension, please refer
-    :func:`~libcst.metadata.Scope.get_qualified_names_for` for more detail.
-    """
-
-    METADATA_DEPENDENCIES = (ScopeProvider,)
-
-    def visit_Module(self, node: cst.Module) -> Optional[bool]:
-        visitor = QualifiedNameVisitor(self)
-        node.visit(visitor)
-
-    @staticmethod
-    def has_name(
-        visitor: MetadataDependent, node: cst.CSTNode, name: Union[str, QualifiedName]
-    ) -> bool:
-        """Check if any of qualified name has the str name or :class:`~libcst.metadata.QualifiedName` name."""
-        qualified_names = visitor.get_metadata(QualifiedNameProvider, node, set())
-        if isinstance(name, str):
-            return any(qn.name == name for qn in qualified_names)
-        else:
-            return any(qn == name for qn in qualified_names)
